==> buckshot-0.6.3.jar: buckshot/worker.clj <==
(ns buckshot.worker
  (:require [buckshot.core :as buckshot]
            [buckshot.queue :as queue]
            [buckshot.util :as util]
            [clojure.stacktrace :as st]))

(defprotocol IWorker
  (start! [this])
  (stop! [this]))

(defn- run-job [{:keys [queue] :as worker} job]
  (buckshot/log! queue {:worker (:id worker)
                        :type "start"
                        :job job})
  (let [f (get (:fns worker) (:fn job))
        start (util/now-ms)
        result (apply f (:args job))
        end (util/now-ms)]
    (queue/publish! queue (:id job) result)
    (queue/finish-job! queue job)
    (buckshot/log! queue {:worker (:id worker)
                          :type "finish"
                          :job job
                          :duration (- end start)})))

(defn- log-exception [{:keys [id queue] :as worker} e]
  (let [trace (with-out-str (st/print-stack-trace e))]
    (buckshot/log! queue {:worker id
                          :type "exception"
                          :trace trace})))

(defrecord Worker [id error-handler fns queue sleep active?]
  IWorker
  (start! [this]
    (when-not @active?
      (reset! active? true)
      (future
        (while @active?
          (try (if-let [j (queue/next-job queue (keys fns))]
                 (if (queue/take-job! queue j)
                   (do (run-job this j)
                       (Thread/sleep 500))
                   (Thread/sleep (rand-int 1000)))
                 (Thread/sleep sleep))
               (catch Throwable t
                 (log-exception t)
                 (when error-handler
                   (try (error-handler t)
                        (catch Throwable t2
                          (log-exception t2))))
                 (Thread/sleep 500)))))))
  (stop! [_]
    (reset! active? false)))

(defn make [{:keys [id error-handler fns queue sleep] :as params}]
  (let [w (Worker. id error-handler fns queue (or sleep 500) (atom false))]
    (start! w)
    w))

==> buckshot-0.6.5.jar: buckshot/worker.clj <==
(ns buckshot.worker
  (:require [buckshot.core :as buckshot]
            [buckshot.queue :as queue]
            [buckshot.util :as util]
            [clojure.stacktrace :as st]))

(defprotocol IWorker
  (start! [this])
  (stop! [this]))

(defn- run-job [{:keys [queue] :as worker} job]
  (buckshot/log! queue {:worker (:id worker)
                        :type "start"
                        :job job})
  (let [f (get (:fns worker) (:fn job))
        start (util/now-ms)
        result (apply f (:args job))
        end (util/now-ms)]
    (queue/publish! queue (:id job) result)
    (queue/finish-job! queue job)
    (buckshot/log! queue {:worker (:id worker)
                          :type "finish"
                          :job job
                          :duration (- end start)})))

(defn- log-exception [{:keys [id queue] :as worker} e]
  (let [trace (with-out-str (st/print-stack-trace e))]
    (buckshot/log! queue {:worker id
                          :type "exception"
                          :trace trace})))

(defrecord Worker [id error-handler fns queue sleep active?]
  IWorker
  (start! [this]
    (when-not @active?
      (reset! active? true)
      (future
        (while @active?
          (try (if-let [j (queue/next-job queue (keys fns))]
                 (if (queue/take-job! queue j)
                   (do (run-job this j)
                       (Thread/sleep 500))
                   (Thread/sleep (rand-int 1000)))
                 (Thread/sleep sleep))
               (catch Throwable t
                 (log-exception this t)
                 (when error-handler
                   (try (error-handler t)
                        (catch Throwable t2
                          (log-exception this t2))))
                 (Thread/sleep 500)))))))
  (stop! [_]
    (reset! active? false)))

(defn make [{:keys [id error-handler fns queue sleep] :as params}]
  (let [w (Worker. id error-handler fns queue (or sleep 500) (atom false))]
    (start! w)
    w))
